/*

  This file is a part of GRALESHELL, a program to facilitate the simulation
  and inversion of gravitational lenses, based on the GRALE library.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "cmdcalc.h"
#include "graleshellinstance.h"
#include "unittypes.h"
#include <shellp/iosystem.h>
#include <shellp/shellcmdrealarg.h>
#include <grale/constants.h>
#include <stdio.h>

#include "debugnew.h"

using namespace shellp;

CmdCalcCriticalDensity::CmdCalcCriticalDensity(const std::string &cmdName) : ShellCommand(cmdName)
{
	m_pDdArg = new ShellCmdRealArg("Dd");
	m_pDdArg->setDescription("Angular diameter distance from observer to lens.");
	m_pDdArg->setMin(0);
	m_pDdArg->setUnitType(UNIT_DISTANCE);

	addArgument(m_pDdArg);

	m_pDsArg = new ShellCmdRealArg("Ds");
	m_pDsArg->setDescription("Angular diameter distance from observer to source.");
	m_pDsArg->setMin(0);
	m_pDsArg->setUnitType(UNIT_DISTANCE);
	addArgument(m_pDsArg);

	m_pDdsArg = new ShellCmdRealArg("Dds");
	m_pDdsArg->setDescription("Angular diameter distance from lens to source.");
	m_pDdsArg->setMin(0);
	m_pDdsArg->setUnitType(UNIT_DISTANCE);
	addArgument(m_pDdsArg);

	setDescription("Calculate the critical density for the specified distances.");
}

CmdCalcCriticalDensity::~CmdCalcCriticalDensity()
{
	delete m_pDdArg;
	delete m_pDsArg;
	delete m_pDdsArg;
}

bool CmdCalcCriticalDensity::execute()
{
	GraleShellInstance *pGSInst = (GraleShellInstance *)ShellInstance::getInstance();
	IOSystem *pIOSys = pGSInst->getIOSystem();

	double Dd = m_pDdArg->getValue();
	double Ds = m_pDsArg->getValue();
	double Dds = m_pDdsArg->getValue();

	double critDens = ((SPEED_C*SPEED_C)/(4.0*CONST_PI*CONST_G*Dd))*(Ds/Dds);
	double massUnit = pGSInst->GetMassUnit();
	double distanceUnit = pGSInst->GetDistanceUnit();
	double critDens2 = critDens*(distanceUnit*distanceUnit)/massUnit;
	char str[1024];
	
	sprintf(str, "Critical density = %g kg m^-2", critDens);
	pIOSys->writeOutputLine(str);
	sprintf(str, "                 = %g (current units)", critDens2);
	pIOSys->writeOutputLine(str);

	return true;
}

