/*

  This file is a part of GRALESHELL, a program to facilitate the simulation
  and inversion of gravitational lenses, based on the GRALE library.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include <mpi.h>
#include "cmdlensplanempi.h"
#include "graleshellinstance.h"
#include "graleshelllensplane.h"
#include "mainmpi.h"

#include "debugnew.h"

using namespace shellp;

CmdLensPlaneMPI::CmdLensPlaneMPI(const std::string &cmdname) : CmdLensPlane(cmdname)
{
	setDescription("Create lens plane based upon the current lens. Calculations are done distributed using MPI.");
}

CmdLensPlaneMPI::~CmdLensPlaneMPI()
{
}

bool CmdLensPlaneMPI::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	grale::GravitationalLens *lens;
	
	if ((lens = gsinst->GetLens()) == 0)
	{
		setErrorString("A lens must have been created first");
		return false;
	}

	// Send the command and transmit the parameters to the other ranks
	int command = GRALESHELL_MPICOMMAND_LENSPLANE;
	MPI_Bcast(&command, 1, MPI_INT, 0, MPI_COMM_WORLD);

	grale::Vector2D<double> bottomleft(GetX1(),GetY1());
	grale::Vector2D<double> topright(GetX2(),GetY2());
	
	GraleShellLensPlane *lensplane = new GraleShellLensPlane(lens);

	if (!lensplane->initMPI(bottomleft,topright,GetNumX(),GetNumY()))
	{
		setErrorString(std::string("Couldn't initialize the lens plane: ") + lensplane->getErrorString());
		delete lensplane;
		return false;
	}
	
	gsinst->SetLensPlane(lensplane);
	
	return true;
}

