/*

  This file is a part of GRALESHELL, a program to facilitate the simulation
  and inversion of gravitational lenses, based on the GRALE library.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "cmdunits.h"
#include "graleshellinstance.h"
#include <shellp/iosystem.h>
#include <shellp/shellcmdrealarg.h>
#include <shellp/shellcmdstringarg.h>
#include <grale/constants.h>

#include "debugnew.h"

using namespace shellp;

CmdUnitMassShow::CmdUnitMassShow(const std::string &cmdname) : ShellCommand(cmdname)
{
	setDescription("Display information about the unit of mass.");
}

CmdUnitMassShow::~CmdUnitMassShow()
{
}

bool CmdUnitMassShow::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	IOSystem *iosys = gsinst->getIOSystem();
	double massunit = gsinst->GetMassUnit();
	
	iosys->writeOutputLine("");
	iosys->writeOutputLine("MASS UNIT:");
	iosys->print(" %g M_sun",(double)(massunit/MASS_SOLAR));
	iosys->print(" %g kg",(double)massunit);
	iosys->writeOutputLine("");
	
	return true;
}

CmdUnitAngularShow::CmdUnitAngularShow(const std::string &cmdname) : ShellCommand(cmdname)
{
	setDescription("Display information about the angular unit.");
}

CmdUnitAngularShow::~CmdUnitAngularShow()
{
}

bool CmdUnitAngularShow::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	IOSystem *iosys = gsinst->getIOSystem();
	double angularunit = gsinst->GetAngularUnit();
	
	iosys->writeOutputLine("");
	iosys->writeOutputLine("ANGULAR UNIT:");
	iosys->print(" %g arcsec",(double)(angularunit/ANGLE_ARCSEC));
	iosys->print(" %g arcmin",(double)(angularunit/ANGLE_ARCMIN));
	iosys->print(" %g deg",(double)(angularunit/ANGLE_DEGREE));
	iosys->print(" %g rad",(double)angularunit);
	iosys->writeOutputLine("");
	
	return true;
}

CmdUnitDistanceShow::CmdUnitDistanceShow(const std::string &cmdname) : ShellCommand(cmdname)
{
	setDescription("Display information about the unit of distance.");
}

CmdUnitDistanceShow::~CmdUnitDistanceShow()
{
}

bool CmdUnitDistanceShow::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	IOSystem *iosys = gsinst->getIOSystem();
	double distunit = gsinst->GetDistanceUnit();
	
	iosys->writeOutputLine("");
	iosys->writeOutputLine("DISTANCE UNIT:");
	iosys->print(" %g Gpc",(double)(distunit/DIST_GPC));
	iosys->print(" %g Mpc",(double)(distunit/DIST_MPC));
	iosys->print(" %g kpc",(double)(distunit/DIST_KPC));
	iosys->print(" %g pc",(double)(distunit/DIST_PC));
	iosys->print(" %g m",(double)distunit);
	iosys->writeOutputLine("");
	
	return true;
}

CmdUnitSet::CmdUnitSet(const std::string &cmdname) : ShellCommand(cmdname)
{
	multarg = new ShellCmdRealArg("unitmultiplier");
	multarg->setMin(0);
	multarg->setDescription("Multiplier for the base unit.");
	addArgument(multarg);
}

CmdUnitSet::~CmdUnitSet()
{
	delete multarg;
}

CmdUnitMassSet::CmdUnitMassSet(const std::string &cmdname) : CmdUnitSet(cmdname)
{
	basearg = new ShellCmdStringArg("unitbase");
	basearg->setDescription("Base unit. Can be either 'kg' or 'M_sun'.");
	addArgument(basearg);
	setDescription("Set the unit of mass to be used when entering commands.");
}

CmdUnitMassSet::~CmdUnitMassSet()
{
	delete basearg;
}

bool CmdUnitMassSet::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	double base = 0;
	
	std::string basestr = basearg->getValue();
	if (basestr == std::string("kg"))
		base = 1.0;
	else if (basestr == std::string("M_sun"))
		base = MASS_SOLAR;
	else
	{
		setErrorString(std::string("The specified base unit '") + basestr + std::string("' is not valid"));
		return false;
	}
	double val = GetMultiplier()*base;
	gsinst->SetMassUnit(val);
	return true;
}

CmdUnitAngularSet::CmdUnitAngularSet(const std::string &cmdname) : CmdUnitSet(cmdname)
{
	basearg = new ShellCmdStringArg("unitbase");
	basearg->setDescription("Base unit. Can be one of 'rad', 'deg', 'arcmin' or 'arcsec'.");
	addArgument(basearg);
	setDescription("Set the angular unit to be used when entering commands.");
}

CmdUnitAngularSet::~CmdUnitAngularSet()
{
	delete basearg;
}

bool CmdUnitAngularSet::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	double base = 0;
	
	std::string basestr = basearg->getValue();
	if (basestr == std::string("rad"))
		base = 1.0;
	else if (basestr == std::string("deg"))
		base = ANGLE_DEGREE;
	else if (basestr == std::string("arcmin"))
		base = ANGLE_ARCMIN;
	else if (basestr == std::string("arcsec"))
		base = ANGLE_ARCSEC;
	else
	{
		setErrorString(std::string("The specified base unit '") + basestr + std::string("' is not valid"));
		return false;
	}
	double val = GetMultiplier()*base;
	gsinst->SetAngularUnit(val);
	return true;
}

CmdUnitDistanceSet::CmdUnitDistanceSet(const std::string &cmdname) : CmdUnitSet(cmdname)
{
	basearg = new ShellCmdStringArg("unitbase");
	basearg->setDescription("Base unit. Can be one of 'm', 'pc', 'kpc', 'Mpc' or 'Gpc'.");
	addArgument(basearg);
	setDescription("Set the unit of distance to be used when entering commands.");
}

CmdUnitDistanceSet::~CmdUnitDistanceSet()
{
	delete basearg;
}

bool CmdUnitDistanceSet::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	double base = 0;
	
	std::string basestr = basearg->getValue();
	if (basestr == std::string("m"))
		base = 1.0;
	else if (basestr == std::string("pc"))
		base = DIST_PC;
	else if (basestr == std::string("kpc"))
		base = DIST_KPC;
	else if (basestr == std::string("Mpc"))
		base = DIST_MPC;
	else if (basestr == std::string("Gpc"))
		base = DIST_GPC;
	else
	{
		setErrorString(std::string("The specified base unit '") + basestr + std::string("' is not valid"));
		return false;
	}
	double val = GetMultiplier()*base;
	gsinst->SetDistanceUnit(val);
	return true;
}

